# -*- coding: utf-8 -*-
from django.http.response import JsonResponse
from django.views.generic.base import View


class ToggleGridView(View):

    def post(self, request, *args, **kwargs):
        # If a post is placed here, toggle the layouter_grid in the session.
        # This causes the button in the toolbar to be active or inactive and the grid to be shown or hidden.
        request.session['layouter_grid'] = not bool(request.session.get('layouter_grid'))
        # Return the result as JSON.
        res = {'show_grid': request.session['layouter_grid']}
        return JsonResponse(res, status=200)



# -*- coding: utf-8 -*-
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from cms.constants import RIGHT


@toolbar_pool.register
class LayouterModifier(CMSToolbar):

    def populate(self):
        # classes for scripting and styling
        extra_classes = ['js-layouterMenu', 'layouter-menuButton']
        # Button should have active state if grid is shown
        if self.request.session.get('layouter_grid'):
            extra_classes.append('cms-btn-active')
        self.toolbar.add_button('Toggle Grid', '', side=RIGHT, extra_classes=extra_classes)



# -*- coding: utf-8 -*-
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _
from cms.plugin_base import CMSPluginBase

from layouter.forms import ContainerPluginForm
from layouter.models import ContainerPlugin


class ContainerCMSPlugin(CMSPluginBase):
    render_template = 'layouter/container.html'
    name = _('Layouting Row')
    model = ContainerPlugin
    allow_children = True
    form = ContainerPluginForm
    change_form_template = 'layouter/change_form.html'

    fieldsets = (
        (None, {
            'fields': ('container_type', 'margin')
        }),
        (_('Advanced'), {
            'classes': ('collapse',),
            'fields': ('css_classes',)
        })
    )

    def render(self, context, instance, placeholder):
        context['width'] = 12 - 2 * instance.margin
        return super(ContainerCMSPlugin, self).render(context, instance, placeholder)

plugin_pool.register_plugin(ContainerCMSPlugin)



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from cms.models.pluginmodel import CMSPlugin
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class ContainerPlugin(CMSPlugin):

    FULL_WIDTH = 100
    THREE_QUARTER_WIDTH = 75
    TWO_THIRD_WIDTH = 66
    HALF_WIDTH = 50
    THIRD_WIDTH = 33
    QUARTER_WIDTH = 25

    NO_MARGIN = _('No margin')
    ONE_COL_MARGIN = '8.33%'
    TWO_COL_MARGIN = '16.66%'
    THREE_COL_MARGIN = '25%'
    FOUR_COL_MARGIN = '33.33%'

    CONTAINER_TYPES = [
        (0, _('One tile - full width')),
        (1, _('Two tiles - 50 | 50')),
        (2, _('Two tiles - 25 | 75')),
        (3, _('Two tiles - 75 | 25')),
        (4, _('Two tiles - 33 | 66')),
        (5, _('Two tiles - 66 | 33')),
        (6, _('Three tiles - 33 | 33 | 33')),
        (7, _('Three tiles - 25 | 25 | 50')),
        (8, _('Three tiles - 25 | 50 | 25')),
        (9, _('Three tiles - 50 | 25 | 25')),
        (10, _('Four tiles - 25 | 25 | 25 | 25')),
    ]

    TYPE_COLUMNS = {
        0: [FULL_WIDTH],
        1: [HALF_WIDTH, HALF_WIDTH],
        2: [QUARTER_WIDTH, THREE_QUARTER_WIDTH],
        3: [THREE_QUARTER_WIDTH, QUARTER_WIDTH],
        4: [THIRD_WIDTH, TWO_THIRD_WIDTH],
        5: [TWO_THIRD_WIDTH, THIRD_WIDTH],
        6: [THIRD_WIDTH, THIRD_WIDTH, THIRD_WIDTH],
        7: [QUARTER_WIDTH, QUARTER_WIDTH, HALF_WIDTH],
        8: [QUARTER_WIDTH, HALF_WIDTH, QUARTER_WIDTH],
        9: [HALF_WIDTH, QUARTER_WIDTH, QUARTER_WIDTH],
        10: [QUARTER_WIDTH, QUARTER_WIDTH, QUARTER_WIDTH, QUARTER_WIDTH]
    }

    MARGIN_TYPES = (
        (0, NO_MARGIN),
        (1, ONE_COL_MARGIN),
        (2, TWO_COL_MARGIN),
        (3, THREE_COL_MARGIN),
        (4, FOUR_COL_MARGIN)
    )

    container_type = models.IntegerField(choices=CONTAINER_TYPES, null=False, blank=False, default=TYPE_COLUMNS[0])

    margin = models.IntegerField(choices=MARGIN_TYPES, null=False, blank=False, default=MARGIN_TYPES[0][0],
                                 help_text=_('How much margin is needed on the left and right side?'))

    css_classes = models.CharField(max_length=512, blank=True, null=True)

    @property
    def max_children(self):
        return len(self.TYPE_COLUMNS[self.container_type])

    def __str__(self):
        name = self.CONTAINER_TYPES[self.container_type][1]
        if self.num_children() > self.max_children:
            name = '<span style="color:red">Warning, too many tiles!</span> {}'.format(name)

        return mark_safe(unicode(name))



# -*- coding: utf-8 -*-
from django import forms
from django.forms.widgets import RadioSelect, ChoiceInput, RadioFieldRenderer
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe

from layouter.models import ContainerPlugin


class ButtonChoiceInput(ChoiceInput):
    input_type = 'radio'

    # This maps the column width to the corresponding css classes.
    FONT_MAPPER = {
        ContainerPlugin.FULL_WIDTH: u'full-width',
        ContainerPlugin.THREE_QUARTER_WIDTH: u'three-quarter',
        ContainerPlugin.TWO_THIRD_WIDTH: u'two-third',
        ContainerPlugin.HALF_WIDTH: u'half',
        ContainerPlugin.THIRD_WIDTH: u'third',
        ContainerPlugin.QUARTER_WIDTH: u'quarter',
    }

    def __init__(self, *args, **kwargs):
        super(ButtonChoiceInput, self).__init__(*args, **kwargs)
        self.value = force_text(self.value)

    def render(self, name=None, value=None, attrs=None):
        if self.choice_value:
            spans = ['<span class="icon-admin {}"></span>'.format(self.FONT_MAPPER[f])
                     for f in ContainerPlugin.TYPE_COLUMNS[int(self.choice_value)]]
            self.choice_label = ' '.join(spans)
            self.choice_label += '</br>' + str(ContainerPlugin.CONTAINER_TYPES[int(self.choice_value)][1])
            self.choice_label = mark_safe(self.choice_label)
        return super(ButtonChoiceInput, self).render(name, value, attrs)


class ButtonSelectRenderer(RadioFieldRenderer):
    choice_input_class = ButtonChoiceInput


class ButtonSelectWidget(RadioSelect):
    renderer = ButtonSelectRenderer


class ContainerPluginForm(forms.ModelForm):
    class Meta:
        model = ContainerPlugin
        fields = ['container_type']
        widgets = {
            'container_type': ButtonSelectWidget
        }

    def clean(self):
        cleaned_data = super(ContainerPluginForm, self).clean()
        cleaned_data['css_classes'] = cleaned_data['css_classes'].strip()
        return cleaned_data



import os
from distutils.core import setup
from layouter import __version__

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Framework :: Django',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Topic :: Internet :: WWW/HTTP',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

REQUIREMENTS = [
    'django>=1.8.14',
    'django-cms>=3.4.0',
]

setup(
    name='djangocms-layouter',
    packages=['layouter'],
    version=__version__,
    description='Grid system for Django-CMS users which aims for ease of use.',
    long_description = read('README.rst'),
    license = read('LICENSE.txt'),
    author='Robert Stein',
    author_email='robert@blueshoe.de',
    url='https://github.com/Blueshoe/djangocms-layouter',
    download_url='https://github.com/Blueshoe/djangocms-layouter/archive/0.1.2.zip',
    install_requires=REQUIREMENTS,
    keywords=['django', 'Django CMS', 'grid', 'bootstrap', 'website', 'CMS', 'Blueshoe'],
    classifiers=CLASSIFIERS,
)
